﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Chessticles
{
	public class King : Piece
	{
		public King(GameModel model, EPlayer owner, Position pos)
			: base(model, EPieceType.King, owner, pos)
		{
		}

		public King(King piece)
			: base(piece)
		{
		}

		public override Piece Clone()
		{
			return new King(this);
		}

		public override void FindValidMoves(List<SMove> moves, int iDir)
		{
			// Test the 8 possible regular moves
			Position[] positions = { new Position(m_position.row + iDir, m_position.col),
															 new Position(m_position.row - iDir, m_position.col),
															 new Position(m_position.row, m_position.col + iDir),
															 new Position(m_position.row, m_position.col - iDir),
															 new Position(m_position.row + iDir, m_position.col + iDir),
															 new Position(m_position.row + iDir, m_position.col - iDir),
															 new Position(m_position.row - iDir, m_position.col + iDir),
															 new Position(m_position.row - iDir, m_position.col - iDir)};
			foreach (Position p in positions)
			{
				if (p.IsValid() && m_model.GetPositionOwner(p) != m_eOwner)
				{
					moves.Add(new SMove(this, new Position(p.row, p.col)));
				}
			}

			// Test for castling
			if (m_iMoveCount == 0 && !m_model.InCheck(m_eOwner))
			{
				FindValidCastling(1, moves);
				FindValidCastling(-1, moves);
			}
		}

		private void FindValidCastling(int iDir, List<SMove> moves)
		{
			// Test if there are empty spaces between King and next piece on the row
			// If that piece is a Rook that hasn't moved, we have a potential castling move
			// (Note that we also require the King doesn't move through a square in check,
			// but that will be tested later during check testing)

			Position pos = new Position(m_position.row, m_position.col + iDir);
			while (pos.IsValid() && m_model.GetPositionOwner(pos) == EPlayer.None)
			{
				pos.col += iDir;
			}
			IPiece piece = m_model.GetPiece(pos);
			if (piece != null && piece.Type == EPieceType.Rook && piece.MoveCount == 0)
			{
				moves.Add(new SMove(this, new Position(m_position.row, m_position.col + 2 * iDir), piece, new Position(m_position.row, m_position.col + iDir)));
			}
		}
	}
}
