﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Chessticles
{
	public class Pawn : Piece
	{
		public Pawn(GameModel model, EPlayer owner, Position pos)
			: base(model, EPieceType.Pawn, owner, pos)
		{
		}

		public Pawn(Pawn piece)
			: base(piece)
		{
		}

		public override Piece Clone()
		{
			return new Pawn(this);
		}

		public override void FindValidMoves(List<SMove> moves, int iDir)
		{
			Position pos = new Position(m_position);

			// Test square in front
			pos.row += iDir;
			if ( pos.IsValid() && m_model.GetPositionOwner(pos) == EPlayer.None )
			{
				// Test if regular move or pawn promotion
				if ((m_eOwner == EPlayer.White && pos.row == 0) ||
						(m_eOwner == EPlayer.Black && pos.row == GameModel.NUM_ROWS - 1))
				{
					moves.Add(new SMove(this, new Position(pos.row, pos.col), EPieceType.Queen)); // Always promote to queen by default
				}
				else
				{
					moves.Add(new SMove(this, new Position(pos.row, pos.col)));
				}
			}

			// Test two squares in front on first move
			pos.row += iDir;
			if (m_iMoveCount == 0 && pos.IsValid() && m_model.GetPositionOwner(pos) == EPlayer.None)
			{
				moves.Add(new SMove(this, new Position(pos.row, pos.col)));
			}

			// Test piece capture
			EPlayer opponent = (m_eOwner == EPlayer.White) ? EPlayer.Black : EPlayer.White; 
			Position[] positions = { new Position(m_position.row + iDir, m_position.col - 1),
															 new Position(m_position.row + iDir, m_position.col + 1)};
			foreach (Position p in positions)
			{
				if (p.IsValid() && m_model.GetPositionOwner(p) == opponent )
				{
					// Test if regular move or pawn promotion
					if ((m_eOwner == EPlayer.White && p.row == 0) ||
							(m_eOwner == EPlayer.Black && p.row == GameModel.NUM_ROWS - 1))
					{
						moves.Add(new SMove(this, new Position(p.row, p.col), EPieceType.Queen)); // Always promote to queen by default
					}
					else
					{
						moves.Add(new SMove(this, new Position(p.row, p.col)));
					}
				}
			}

			// Test en passant
			SMove lastMove = m_model.LastMove;
			if (lastMove.type == EMoveType.Standard && 
				  lastMove.piece.Type == EPieceType.Pawn &&
				  lastMove.piece.MoveCount == 1 &&
				  m_position.row == lastMove.piece.Position.row &&
				  (m_position.row == 3 || m_position.row == 4) &&
				  Math.Abs( m_position.col - lastMove.piece.Position.col ) == 1)
			{
				moves.Add(new SMove(this, new Position(m_position.row + iDir, lastMove.piece.Position.col), lastMove.piece));
			}
		}
	}
}
